(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else if(typeof exports === 'object')
		exports["taiwanIdValidator"] = factory();
	else
		root["taiwanIdValidator"] = factory();
})(this, function() {
return /******/ (() => { // webpackBootstrap
/******/ 	"use strict";
var __webpack_exports__ = {};
// This entry need to be wrapped in an IIFE because it uses a non-standard name for the exports (exports).
(() => {
var exports = __webpack_exports__;

Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isDonateCode = exports.isCellPhoneBarcode = exports.isCDC = exports.isOriginalRC = exports.isNewRC = exports.isRC = exports.isNI = exports.isGUI = exports.isEInvoiceDonateCodeValid = exports.isEInvoiceCellPhoneBarcodeValid = exports.isCitizenDigitalCertificateNumberValid = exports.isOriginalResidentCertificateNumberValid = exports.isNewResidentCertificateNumberValid = exports.isResidentCertificateNumberValid = exports.isNationalIdentificationNumberValid = exports.isGuiNumberValid = void 0;
/**
 * Verify the input is a valid GUI Number (中華民國統一編號)
 *
 * @param { string | number } input GUI Number
 * @returns { boolean } is `input` a valid GUI number
 */
function isGuiNumberValid(input) {
    if (typeof input !== 'string' && typeof input !== 'number')
        return false;
    /**
     * Example: 12345675
     * Step 1:
     * 1 * 1 = 1
     * 2 * 2 = 4
     * 3 * 1 = 3
     * 4 * 2 = 8
     * 5 * 1 = 5
     * 6 * 2 = 12
     * 7 * 4 = 28
     * 5 * 1 = 5
     *
     * Step 2:
     * 1 -> 1
     * 4 -> 4
     * 3 -> 3
     * 8 -> 8
     * 5 -> 5
     * 12 -> 1 + 2 = 3
     * 28 -> 2 + 8 = 10
     * 5 -> 5
     *
     * Step 3:
     * (1 + 4 + 3 + 8 + 5 + 3 + 10 + 5) % 10 = 9
     */
    var GUI_NUMBER_COEFFICIENTS = [1, 2, 1, 2, 1, 2, 4, 1];
    var n = input.toString();
    var regex = /^\d{8}$/;
    if (!regex.test(n))
        return false;
    /**
     * Step 1: 先把統一編號的每個數字分別乘上對應的係數 (1, 2, 1, 2, 1, 2, 4, 1)
     * Step 2: 再把個別乘積的十位數與個位數相加，得出八個小於 10 的數字
     */
    var checksum = GUI_NUMBER_COEFFICIENTS.reduce(function (sum, c, index) {
        var product = c * parseInt(n.charAt(index), 10);
        return sum + (product % 10) + Math.floor(product / 10);
    }, 0);
    /**
     * Step 3: 檢查把這 8 個數字相加之後計算此和除以 10 的餘數
     * Step 4:
     *  4-1: 若是餘數為 0，則為正確的統一編號
     *  4-2: 若是餘數為 9，且原統一編號的第七位是 7，則也為正確的統一編號
     */
    return (checksum % 10 === 0 ||
        (parseInt(n.charAt(6), 10) === 7 && (checksum + 1) % 10 === 0));
}
exports.isGuiNumberValid = isGuiNumberValid;
/**
 * Verify the input is a valid National identification number (中華民國身分證字號)
 *
 * @param { string } input National identification number
 * @returns { boolean } is `input` a valid national ID number
 */
function isNationalIdentificationNumberValid(input) {
    if (typeof input !== 'string')
        return false;
    var regex = /^[A-Z][1,2]\d{8}$/;
    return regex.test(input) && verifyTaiwanIdIntermediateString(input);
}
exports.isNationalIdentificationNumberValid = isNationalIdentificationNumberValid;
/**
 * Verify the input is a valid resident certificate number (臺灣地區無戶籍國民、外國人、大陸地區人民及香港或澳門居民之專屬代號)
 *
 * @param { string } input resident certificate number
 * @returns { boolean } is `input` a valid resident certificate number
 */
function isResidentCertificateNumberValid(input) {
    if (typeof input !== 'string')
        return false;
    return (isNewResidentCertificateNumberValid(input) ||
        isOriginalResidentCertificateNumberValid(input));
}
exports.isResidentCertificateNumberValid = isResidentCertificateNumberValid;
/**
 * Verify the input is a valid new resident certificate number (臺灣地區無戶籍國民、外國人、大陸地區人民及香港或澳門居民之專屬代號)
 *
 * @param { string } input resident certificate number
 * @returns { boolean } is `input` a valid new resident certificate number
 */
function isNewResidentCertificateNumberValid(input) {
    if (typeof input !== 'string')
        return false;
    var regex = /^[A-Z][8,9]\d{8}$/;
    return regex.test(input) && verifyTaiwanIdIntermediateString(input);
}
exports.isNewResidentCertificateNumberValid = isNewResidentCertificateNumberValid;
/**
 * Verify the input is a original valid resident certificate number (臺灣地區無戶籍國民、外國人、大陸地區人民及香港或澳門居民之專屬代號)
 *
 * @param { string } input resident certificate number
 * @returns { boolean } is `input` a valid original resident certificate number
 */
function isOriginalResidentCertificateNumberValid(input) {
    if (typeof input !== 'string')
        return false;
    var regex = /^[A-Z]{2}\d{8}$/;
    return regex.test(input) && verifyTaiwanIdIntermediateString(input);
}
exports.isOriginalResidentCertificateNumberValid = isOriginalResidentCertificateNumberValid;
/**
 * Verify the input is a valid citizen digital certificate number (自然人憑證)
 *
 * @param { string } input citizen digital certificate number
 * @returns { boolean } is `input` a valid citizen digital certificate number
 */
function isCitizenDigitalCertificateNumberValid(input) {
    if (typeof input !== 'string')
        return false;
    var n = input.toString();
    // 驗證規則為兩碼英文 + 14 碼數字
    var regex = /^[A-Z]{2}\d{14}$/;
    return regex.test(n);
}
exports.isCitizenDigitalCertificateNumberValid = isCitizenDigitalCertificateNumberValid;
/**
 * Verify the input is a valid E-Invoice cell phone barcode (電子發票手機條碼)
 *
 * @param { string } input E-Invoice cell phone barcode
 * @returns { boolean } is `input` a valid e-invoice cell phone barcode
 */
function isEInvoiceCellPhoneBarcodeValid(input) {
    if (typeof input !== 'string')
        return false;
    var n = input.toString();
    /**
     * 總長度為 8 碼
     * 第 1 碼為 /
     * 第 2-8 碼由 0-9 (數字), A-Z (大寫英文字母), .(period), -(hyphen), +(plus) 組成
     */
    var regex = /^\/[\dA-Z.\-+]{7}$/;
    return regex.test(n);
}
exports.isEInvoiceCellPhoneBarcodeValid = isEInvoiceCellPhoneBarcodeValid;
/**
 * Verify the input is a valid E-Invoice donate code (電子發票捐贈碼)
 *
 * @param { string | number } input E-Invoice donate code
 * @returns { boolean } is `input` a valid e-invoice donate code
 */
function isEInvoiceDonateCodeValid(input) {
    if (typeof input !== 'string' && typeof input !== 'number')
        return false;
    var n = input.toString();
    // 總長度為 3-7 碼 0-9 的數字
    var regex = /^[\d]{3,7}$/;
    return regex.test(n);
}
exports.isEInvoiceDonateCodeValid = isEInvoiceDonateCodeValid;
/**
 * Verify the intermediate string for isNationalIdentificationNumberValid and isResidentCertificateNumberValid
 *
 * @param { string } input String to verify
 * @returns { boolean } is `input` a valid Taiwan ID intermediate string
 */
function verifyTaiwanIdIntermediateString(input) {
    var idArray = input.split('');
    var intRadix = 10;
    /**
     *  A=10 台北市     J=18 新竹縣     S=26 高雄縣
     *  B=11 台中市     K=19 苗栗縣     T=27 屏東縣
     *  C=12 基隆市     L=20 台中縣     U=28 花蓮縣
     *  D=13 台南市     M=21 南投縣     V=29 台東縣
     *  E=14 高雄市     N=22 彰化縣     W=32 金門縣*
     *  F=15 台北縣     O=35 新竹市*    X=30 澎湖縣
     *  G=16 宜蘭縣     P=23 雲林縣     Y=31 陽明山
     *  H=17 桃園縣     Q=24 嘉義縣     Z=33 連江縣*
     *  I=34 嘉義市*    R=25 台南縣
     *
     *  Step 1: 英文字母按照上表轉換為數字之後，十位數 * 1 + 個位數 * 9 相加
     */
    var TAIWAN_ID_LOCALE_CODE_LIST = [
        1,
        10,
        19,
        28,
        37,
        46,
        55,
        64,
        39,
        73,
        82,
        2,
        11,
        20,
        48,
        29,
        38,
        47,
        56,
        65,
        74,
        83,
        21,
        3,
        12,
        30 // Z -> 33 -> 1 * 3 + 9 * 3 = 30
    ];
    var RESIDENT_CERTIFICATE_NUMBER_LIST = [
        '0',
        '1',
        '2',
        '3',
        '4',
        '5',
        '6',
        '7',
        '4',
        '8',
        '9',
        '0',
        '1',
        '2',
        '5',
        '3',
        '4',
        '5',
        '6',
        '7',
        '8',
        '9',
        '2',
        '0',
        '1',
        '3' // Z
    ];
    // if is not a number (舊版居留證編號)
    if (isNaN(parseInt(idArray[1], intRadix))) {
        idArray[1] =
            RESIDENT_CERTIFICATE_NUMBER_LIST[input.charCodeAt(1) - 'A'.charCodeAt(0)];
    }
    // Step 2: 第 1 位數字 (只能為 1 or 2) 至第 8 位數字分別乘上 8, 7, 6, 5, 4, 3, 2, 1 後相加，再加上第 9 位數字
    var cb = function (sum, n, index) {
        return sum +
            (index === 0
                ? TAIWAN_ID_LOCALE_CODE_LIST[n.charCodeAt(0) - 'A'.charCodeAt(0)]
                : parseInt(n, intRadix) * (index === 9 ? 1 : 9 - index));
    };
    // Step 3: 如果該數字為 10 的倍數，則為正確身分證字號
    return idArray.reduce(cb, 0) % 10 === 0;
}
exports.isGUI = isGuiNumberValid;
exports.isNI = isNationalIdentificationNumberValid;
exports.isRC = isResidentCertificateNumberValid;
exports.isNewRC = isNewResidentCertificateNumberValid;
exports.isOriginalRC = isOriginalResidentCertificateNumberValid;
exports.isCDC = isCitizenDigitalCertificateNumberValid;
exports.isCellPhoneBarcode = isEInvoiceCellPhoneBarcodeValid;
exports.isDonateCode = isEInvoiceDonateCodeValid;

})();

/******/ 	return __webpack_exports__;
/******/ })()
;
});
//# sourceMappingURL=index.map